/************************************************************************/
/*								INCLUDE FILES							*/
/************************************************************************/
#include <stdio.h>
#include "can.h"
/************************************************************************/
/*					Initialisierung von SPI und MCP2515					*/
/************************************************************************/
bool can_init(uint8 bitrate)
{
	#ifdef _MSC_VER
		Init_libMPSSE();
	#endif

	uint32 spi_channel = 0;			// Kanal waehlen den man oeffnen moechte
	uint32 spi_clockrate = 50000;	// Geschwindigkeit der Datenuebertragung waehlen (5000 = 10kHz);
	
	if (spi_init(spi_channel, spi_clockrate))
	{
		spi_write(SPI_RESET);
		Sleep(1);
		Reset_CS();
		
		Sleep(10);

		if(mcp2515_init(bitrate))
			return true;
	}
	return false;
}
/************************************************************************/
/*							Kanal schlieen								*/
/************************************************************************/
void can_close(void)
{
	spi_closeChannel();

	#ifdef _MSC_VER
		Cleanup_libMPSSE();
	#endif
}

/************************************************************************/
/*					gesamte	Nachricht empfangen							*/
/************************************************************************/
bool can_get_message(can_t *msg)
{
	uint8 addr;
	uint8 tmp;
	uint8 length;

	// lese Status
	uint8 status = mcp2515_read_status(SPI_RX_STATUS);
		
	if ((status & 0x40) > 0) // Maske um Bit 6 abzufragen
	// Nachricht in Empfangspuffer RXB0
	{	
		addr = SPI_READ_RX;
	}
	else if ((status & 0x80) > 0) // Maske um Bit 7 abzufragen
	// Nachricht in Empfangspuffer RXB1
	{	
		addr = SPI_READ_RX | 0x04;
	}
	else 
	// Fehler: keine Nachricht vorhanden
	{
		// Fehler: keine Nachricht vorhanden
		return false;
	}
	
	spi_write(addr);
	
	// CAN ID auslesen und ueberpruefen
	tmp = mcp2515_read_id(&msg->id);
	
	// fragt Extended ID Bit ab
	// und setzt oder loescht entsprechend Extended ID flag
	msg->flags.extended = tmp & 0x01;
	
	// liest Datenlaenge (in Byte)
	length = spi_read();

	// fragt RTR bit ab
	// und setzt oder loescht entsprechend RTR flag
	msg->flags.rtr = ((status & 0x08) > 0) ? 1 : 0;  // Maske um Bit 3 abzufragen
	
	// die 4 Laengenbits rauslesen
	length &= 0x0f;

	// und in Nachricht reinschreiben
	msg->length = length;

	// liest Daten
	for (uint8 i = 0; i < length; i++)
	{
		// schreibt die gelesene Daten in Nachricht
		msg->data[i] = spi_read();
	}
	Reset_CS();
	
	// Loesche Interrupt Flag
	if ((status & 0x40) > 0)	// Maske um Bit 6 abzufragen
		mcp2515_bit_modify(CANINTF, (1 << RX0IF), 0);
	else
		mcp2515_bit_modify(CANINTF, (1 << RX1IF), 0);
		
		return true;
}

/************************************************************************/					
/*						Can setzt/loescht RTR Bit						*/
/*				schreibt Laenge und die Daten in Puffer TXn				*/
/************************************************************************/
bool can_send_data(uint8 *data, uint8 length, uint8 rtr_, uint8 puffer)
{
	uint8 temp;

	if (puffer == 0)
	// Nachricht wird in Puffer TX0 geschrieben
	{
		if(rtr_ == 1)
		// wenn rtr 1 setze RTR Bit
			temp = 0x40 | length;
		else
		// sonst loesche RTR Bit
			temp = length;

		// schreibe Laenge und RTR in das Register
		mcp2515_write_register(TXB0DLC, temp);

		if (rtr_ == 0)
		// wenn rtr = 0 ist, ist es keine Request Message
		// d.h. die Nachricht enthlt Laenge und Daten
		{
			spi_write(SPI_WRITE_TX0D);
			for (temp = 0; temp < length; temp++)
			{
				spi_write(data[temp]);
			}
			Reset_CS();
		}

		// sende Nachricht aus Puffer TXB0
		spi_write(SPI_RTS_TXB0);
		Reset_CS();
		return true;
	}
	
	else if (puffer == 1)
	// Nachricht wird in Puffer TX1 geschrieben
	{
		if(rtr_ == 1)
		// wenn rtr 1 setze RTR Bit
			temp = 0x40 | length;
		else
		// sonst loesche RTR Bit
			temp = length;
		
		// schreibe Laenge und RTR in das Register
		mcp2515_write_register(TXB1DLC, temp);

		if (rtr_ == 0)
		// wenn rtr = 0 ist, ist es keine Request Message
		// d.h. die Nachricht enthlt Laenge und Daten
		{
			spi_write(SPI_WRITE_TX1D);
			for (temp = 0; temp < length; temp++)
			{
				spi_write(data[temp]);
			}
			Reset_CS();
		}
		// sende Nachricht aus Puffer TXB1
		spi_write(SPI_RTS_TXB1);
		Reset_CS();
		return true;
	}
	else if (puffer == 2)
		// Nachricht wird in Puffer TX2 geschrieben
	{
		if(rtr_ == 1)
			temp = 0x40 | length;
		else
			temp = length;
		mcp2515_bit_modify(TXB2DLC, 0x4F, temp);

		if (rtr_ == 0)
		// wenn rtr = 0 ist, ist es keine Request Message
		// d.h. die Nachricht enthlt Laenge und Daten
		{
			spi_write(SPI_WRITE_TX2D);
			for (temp = 0; temp < length; temp++)
			{
				spi_write(data[temp]);
			}
			Reset_CS();
		}
		// sende Nachricht aus Puffer TXB2
		spi_write(SPI_RTS_TXB2);
		Reset_CS();
		return true;
	}
	else
		return false;
}

/************************************************************************/
/*						Can ID reinschreiben							*/
/************************************************************************/
bool can_set_id(uint32 id, uint8 extended, uint8 puffer)
{
	if(puffer == 0)
	{
		// ID in Puffer0 schreiben
		spi_write(SPI_WRITE_TX0);
		mcp2515_write_id(&id, extended);

		Reset_CS();
		return true;
	}
	else if(puffer == 1)
	{
		// ID in Puffer1 schreiben
		spi_write(SPI_WRITE_TX1);
		mcp2515_write_id(&id, extended);

		Reset_CS();
		return true;
	}
	else if(puffer == 2)
	{
		// ID in Puffer2 schreiben
		spi_write(SPI_WRITE_TX2);
		mcp2515_write_id(&id, extended);

		Reset_CS();
		return true;
	}
	else
		return false;
}

/************************************************************************/
/*								 Modus setzen							*/
/************************************************************************/
void can_set_mode(can_mode_t mode)
{
	uint8 reg = 0;
	
	if (mode == LISTEN_ONLY_MODE) {
		reg = (1 << REQOP1)|(1 << REQOP0);
	}
	else if (mode == LOOPBACK_MODE) {
		reg = (1 << REQOP1);
	}
	else if (mode == SLEEP_MODE) {
		reg = (1 << REQOP0);
	}
		
	// setze den neuen Modus
	mcp2515_bit_modify(CANCTRL, (1 << REQOP2) | (1 << REQOP1) | (1 << REQOP0), reg);
	while ((mcp2515_read_register(CANSTAT) & 0xe0) != reg) 
	{
		// warte bis der neue Modus aktiv wird
	}
}


/************************************************************************/
/*						Ausgabe Programmbeschreibung					*/
/************************************************************************/
void ausgabe(void)
{
	printf("***********************************************************************\n");
	printf("\t  *\n");
	printf("\t    *         *\n");
	printf("\t      *       *    *\n");
	printf("\t        *     *   *\n");
	printf("\t          *   *  *\n");
	printf("\t            * * * \t\tLessmueller Lasertechnik\n");
	printf("\t              * ** \n");
	printf("\t            * *     *  \n");
	printf("\t              *        *\n");
	printf("\t              *           *\n");
	printf("***********************************************************************\n");
	printf("\nDieses Programm dient dazu eine LED durch Tastendruck einzuschalten.\n");
	printf("Immer, wenn die Taste gedrueckt wird, wird die LED eingeschalten,\n");
	printf("sonst wird die LED ausgeschalten.\n");
	printf("Das Programm startet 20 sec nach Ausfuehrung und wird durch eine\n");
	printf("beliebige Tastatureingabe beendet.\n");

	Sleep(20000);
}
/************************************************************************/
/*								Dateiende								*/
/************************************************************************/