#ifndef CAN_H
#define CAN_H

// -------------------------------------------------------------------------
#include "mcp2515.h"

// -------------------------------------------------------------------------
// Defines

#define RTR_	1

// Puffer
#define TXB_0		0
#define TXB_1		1
#define TXB_2		2

// -------------------------------------------------------------------------
// Bitraten fuer den CAN-Bus 
typedef enum {
	BITRATE_10_KBPS	= 0,	
	BITRATE_20_KBPS	= 1,	
	BITRATE_50_KBPS	= 2,	
	BITRATE_100_KBPS = 3,	
	BITRATE_125_KBPS = 4,
	BITRATE_250_KBPS = 5,	
	BITRATE_500_KBPS = 6,	
	BITRATE_1_MBPS = 7,		
} can_bitrate_t;

// -------------------------------------------------------------------------
// Datenstruktur zum Aufnehmen von CAN Nachrichten
typedef struct
{
	uint32 id;				//!< ID der Nachricht (11 oder 29 Bit)
	struct 
	{
		int rtr : 1;		//!< Remote-Transmit-Request-Frame?
		int extended : 1;	//!< extended ID?
	} flags;
		
	uint8 length;			//!< Anzahl der Datenbytes
	uint8 data[8];			//!< Die Daten der CAN Nachricht
	
} can_t;

// -------------------------------------------------------------------------
// Zustaende des CAN Interfaces
typedef enum 
{
	LISTEN_ONLY_MODE,		//!< der CAN Contoller empfaengt nur und verhaelt sich voellig passiv
	LOOPBACK_MODE,			//!< alle Nachrichten direkt auf die Empfangsregister umleiten ohne sie zu senden
	NORMAL_MODE,			//!< normaler Modus, CAN Controller ist aktiv
	SLEEP_MODE				// sleep mode
} can_mode_t;

// -------------------------------------------------------------------------
// Liest eine Nachricht aus den Empfangspuffern des CAN Controllers
// Parameter: msg	Pointer auf die Nachricht die gelesen werden soll.
// Rueckgabewert: false falls keine Nachricht erhalten, sonst true
extern bool can_get_message(can_t *msg);

// ----------------------------------------------------------------------------
// Setzt den Operations-Modus
// Parameter: mode	Gewuenschter Modus des CAN Controllers
extern void can_set_mode(can_mode_t mode);

// ----------------------------------------------------------------------------
// schreibt ID in ausgewhlten TXBn Puffer
// Paramater: ID		die reingeschrieben werden soll
//			  extended	ob ID standard oder extended
//			  puffer	gibt an in welchen Puffer die ID geschrieben wird
extern bool can_set_id(uint32 id, uint8 extended, uint8 puffer);

// ----------------------------------------------------------------------------
// schreibt Daten in Senderegister und schickt die entsprechnenden Register raus
// Parameter: data		Pointer auf die Daten gesendet werden sollen
//			  length	Anzahl der Datenbytes
//			  rtr		einstellen ob Request Nachricht oder nicht
//			  puffer	gibt an welcher Puffer beschrieben und gesendet werden soll
extern bool can_send_data(uint8 *data, uint8 length, uint8 rtr_, uint8 puffer);

// ----------------------------------------------------------------------------
// Initialisierung von CAN
// Parameter: bitrate  Einstellung in welche Geschwindigkeit CAN Daten sendet
// Rckgabewert: bool false falls bei der Initialisierung etwas schief gelaufen ist,
//				 ansonsten true.
extern bool can_init(uint8 bitrate);

// ----------------------------------------------------------------------------
extern void can_close(void);

// ----------------------------------------------------------------------------
// gibt einen Text aus
void ausgabe(void);

#endif