/************************************************************************/
/*								INCLUDE FILES							*/
/************************************************************************/
#include "mcp2515.h"


#ifndef	MCP2515_CLKOUT_PRESCALER
	#error	MCP2515_CLKOUT_PRESCALER not defined!
#elif MCP2515_CLKOUT_PRESCALER == 0
	#define	CLKOUT_PRESCALER_	0x0
#elif MCP2515_CLKOUT_PRESCALER == 1
	#define	CLKOUT_PRESCALER_	0x4
#elif MCP2515_CLKOUT_PRESCALER == 2
	#define	CLKOUT_PRESCALER_	0x5
#elif MCP2515_CLKOUT_PRESCALER == 4
	#define	CLKOUT_PRESCALER_	0x6
#elif MCP2515_CLKOUT_PRESCALER == 8
	#define	CLKOUT_PRESCALER_	0x7
#else
	#error	invaild value of MCP2515_CLKOUT_PRESCALER
#endif

/************************************************************************/
/*					MCP2515	schreibt in die Register					*/
/************************************************************************/
void mcp2515_write_register(uint8 adress, uint8 data)
{
	spi_write(SPI_WRITE);
	spi_write(adress);
	spi_write(data);
	Reset_CS();
}

/************************************************************************/
/*					MCP2515	liest aus den Registern						*/
/************************************************************************/
uint8 mcp2515_read_register(uint8 adress)
{
	uint8 data;

	spi_write(SPI_READ);
	spi_write(adress);

	data = spi_read();
	
	Reset_CS();
		
	return data;
}

/************************************************************************/
/*						MCP2515	liest Status							*/
/************************************************************************/
uint8 mcp2515_read_status(uint8 type)
{
	uint8 data;
		
	spi_write(type);
	data = spi_read();
	
	Reset_CS();
		
	return data;
}

/************************************************************************/
/*					MCP2515	loescht/setzt einzelne Bits					*/
/************************************************************************/
void mcp2515_bit_modify(uint8 adress, uint8 mask, uint8 data)
{
	spi_write(SPI_BIT_MODIFY);
	spi_write(adress);
	spi_write(mask);
	spi_write(data);
	
	Reset_CS();
}

/************************************************************************/
/*							Bitrateeinstellung							*/
/*					  bernommen von Fabian Greif						*/
/************************************************************************/
uint8 _mcp2515_cnf[8][3] = 
{
	// 10 kbps
	{	0x04,
		0xb6,
		0xe7
	},
	// 20 kbps
	{	0x04,
		0xb6,
		0xd3
	},
	// 50 kbps
	{	0x04,
		0xb6,
		0xc7
	},
	// 100 kbps
	{	0x04,
		0xb6,
		0xc3
	},
	// 125 kbps
	{	(1 << PHSEG21),					// CNF3
		(1 << BTLMODE) | (1 << PHSEG11),		// CNF2
		(1 << BRP2) | (1 << BRP1) | (1 << BRP0)	// CNF1
	},
	// 250 kbps
	{	0x03,
		0xac,
		0x81
	},
	// 500 kbps
	{	0x03,
		0xac,
		0x80
	},
	// 1 Mbps
	{	(1 << PHSEG21),
		(1 << BTLMODE) | (1 << PHSEG11),
		0
	}
};

/************************************************************************/
/*								Init MCP2515							*/
/************************************************************************/
bool mcp2515_init(uint8 bitrate)
{
	if (bitrate >= 8)
		return false;
	
	
	// MCP2515 per Software Reset zuruecksetzten,
	// danach ist er automatisch im Konfigurations Modus
	spi_write(SPI_RESET);

	Sleep(1);
	
	Reset_CS();
	
	// ein bisschen warten bis der MCP2515 sich neu gestartet hat
	Sleep(10);
	
	// CNF1..3 Register laden (Bittiming)
	spi_write(SPI_WRITE);
	spi_write(CNF3);
	for (uint8 i = 0; i < 3 ; i++) 
	{
		spi_write(_mcp2515_cnf[bitrate][i]);
	}

	// aktivieren/deaktivieren der Interrupts
	spi_write(MCP2515_INTERRUPTS);
	Reset_CS();
	
	// TXnRTS Bits als Inputs schalten
	mcp2515_write_register(TXRTSCTRL, 0);
	
	// Deaktivieren der Pins RXnBF Pins (High Impedance State)
	mcp2515_write_register(BFPCTRL, 0);

	// Testen ob das auf die beschreibenen Register zugegriffen werden kann
	// (=> ist der Chip ueberhaupt ansprechbar?)
	bool error = false;
	if (mcp2515_read_register(CNF2) != _mcp2515_cnf[bitrate][1]) 
	{
		error = true;
	}
	// Device zurueck in den normalen Modus versetzten
	// und aktivieren/deaktivieren des Clkout-Pins
	mcp2515_write_register(CANCTRL, CLKOUT_PRESCALER_);
	
	if (error) 
	{
		return false;
	}
	else
	{
		while ((mcp2515_read_register(CANSTAT) & 0xe0) != 0) 
		{
			// warten bis der neuer Modus aktiv ist
		}
		return true;
	}
}

/************************************************************************/
/*						liest ID aus den Registern					*/
/************************************************************************/
uint8 mcp2515_read_id(uint32 *id)
{
	uint8 first;
	uint8 tmp;
	uint8 wait;
	
	// liest Standard ID aus
	// und speichert sie in Variable
	first = spi_read();		// Standard ID Bit 10-3
	tmp   = spi_read();		// Standard ID Bit 2-0 und Extended ID 17-16
	
	if (tmp & (1 << IDE)) 
	// Nachtricht hat eine Extended ID
	{
		// liest Extended ID aus
		// und speichert sie in Variable
		wait = spi_read();	// Extended ID Bit 15-8
		
		// rechnet ID um
		*((uint16 *) id + 1)  = (uint16) first << 5;
		*((uint8 *)  id + 1)  = wait;
		
		// liest Extended ID aus
		// und speichert sie in Variable
		wait = spi_read();	// Extended ID Bit 7-0 
		
		// rechnet ID um
		*((uint8 *)  id + 2) |= (tmp >> 3) & 0x1C;
		*((uint8 *)  id + 2) |=  tmp & 0x03;
		
		*((uint8 *)  id)      = wait;
		
		return TRUE;
	}
	else 
	// Nachricht hat nur eine Standard ID
	{
		// liest Extended ID aus und verwirft sie
		spi_read();

		// rechnet ID um
		*((uint8 *)  id + 3) = 0;
		*((uint8 *)  id + 2) = 0;
		
		*((uint16 *) id) = (uint16) first << 3;
		
		Sleep(1);

		// liest Extended ID aus und verwirft sie
		spi_read();
		
		// rechnet ID um
		*((uint8 *) id) |= tmp >> 5;
		
		Sleep(1);
		
		return FALSE;
	}
}

/************************************************************************/
/*							Hilfsanweisung								*/
/************************************************************************/

static uint8 usi_interface_spi_temp;

static void spi_start(uint8 data) 
{
	usi_interface_spi_temp = data;
	spi_write(data);
}

static uint8 spi_wait(void) 
{
	return usi_interface_spi_temp;
}

/************************************************************************/
/*					 schreibt die ID in die Registern					*/
/************************************************************************/
void mcp2515_write_id(const uint32 *id, uint8 extended)
{
	uint8 tmp;
	
	if (extended) 
	{
		// ID umwandeln und ins Register schreiben
		// Standard ID Bit 10-3
		spi_start(*((uint16 *) id + 1) >> 5);
		
		// naechsten Werte berechnen
		tmp  = (*((uint8 *) id + 2) << 3) & 0xe0;
		tmp |= (1 << IDE);
		tmp |= (*((uint8 *) id + 2)) & 0x03;
		
		// warten bis der vorherige Werte geschrieben wurde
		spi_wait();
		
		// restliche Werte schreiben
		// Standard ID Bit 2-0 und Extended ID Bit 17-16
		spi_write(tmp);
		// Extended ID Bit 15-8
		spi_write(*((uint8 *) id + 1));
		// Extended ID Bit 7-0
		spi_write(*((uint8 *) id));
	}
	else 
	{
		// ID umwandeln und ins Register schreiben
		// Standard ID Bit 10 - 3
		spi_start(*((uint16 *) id) >> 3);
		
		// naechsten Werte berechnen
		tmp = *((uint8 *) id) << 5;
		spi_wait();
		
		// restliche umgewandelte Werte ins Register schreiben
		// Standard ID Bit 2-0
		spi_write(tmp);
		// schreibt fuer Extended ID 0
		spi_write(0);
		spi_write(0);
	}
}

/************************************************************************/
/*				 sendet Nachricht aus TXn Puffer						*/
/*		buffer = SPI_RTS_TXBn; where 'n' = 0, 1, 2						*/
/*		buffer = SPI_RTS; followed by | 0 - 7 (e.g., CAN_RTS | 7)		*/
/************************************************************************/
void RTS(unsigned char buffer)
{
	uint8 tempdata;

	//liest Status einzelner Bits von verschiedenen Registern
	tempdata = mcp2515_read_status(SPI_READ_STATUS);
	
	if (buffer & 0x01)			//Puffer 0	
		if (tempdata & 0x04)	//Ueberpruefe erst TXREQ
		{
			Sleep(1);

			//Loesche TXREQ (und alles andere... was nicht geloescht ist)
			mcp2515_write_register(TXB0CTRL, 0);

			//Warte solange bis TXREQ geloescht ist
			while(mcp2515_read_status(SPI_READ_STATUS) & 0x04);
		}

	if (buffer & 0x02)			//Puffer 1
		if (tempdata & 0x10)	//Ueberpruefe erst TXREQ
		{
  			Sleep(1);

			//Loesche TXREQ (und alles andere... was nicht geloescht ist)
			mcp2515_write_register(TXB1CTRL, 0);

			//Warte solange bis TXREQ geloescht ist
			while(mcp2515_read_status(SPI_READ_STATUS) & 0x10);	
		}

	if (buffer & 0x04)			//Puffer 2
		if (tempdata & 0x40)	//Ueberpruefe erst TXREQ
		{
			Sleep(1);

			//Loesche TXREQ (und alles andere... was nicht geloescht ist)
			mcp2515_write_register(TXB2CTRL, 0);

			//Warte solange bis TXREQ geloescht ist
			while(mcp2515_read_status(SPI_READ_STATUS) & 0x40);	
		}
		
	// sendet Nachricht aus entsprechenden Puffer raus 
	spi_write(buffer);

	Reset_CS();
}

/************************************************************************/
/*								Dateiende								*/
/************************************************************************/