#ifndef MCP2515_H
#define MCP2515_H

// -------------------------------------------------------------------------
#include"spi.h"
#include"mcp2515_defs.h"

// -------------------------------------------------------------------------
// laedt einige Standardwerte
#ifndef	MCP2515_CLKOUT_PRESCALER
	#define	MCP2515_CLKOUT_PRESCALER	0
#endif

#ifndef	MCP2515_INTERRUPTS
	#define	MCP2515_INTERRUPTS			(1<<RX1IE)|(1<<RX0IE)
#endif

// -------------------------------------------------------------------------
// Initialisierung des CAN Interfaces
// Parameter: bitrate	Gewuenschte Geschwindigkeit des CAN Interfaces
// Rueckgabewert: false falls das CAN Interface nicht initialisiert werden konnte,
//				 ansonsten true.
extern bool mcp2515_init(uint8 bitrate);

// -------------------------------------------------------------------------
// Beschreiben von internen Registern
// Parameter: adres Adresse vom Register, in das geschrieben werden soll
//			  data  uebergibt was reingeschrieben werden soll
extern void mcp2515_write_register(uint8 adress, uint8 data);

// -------------------------------------------------------------------------
// Lesen der internen Register
// Parameter: adres Register aus dem gelesen werden soll
//			  type welcher Status abgefragt werden soll
// Rueckgabewert: die Daten die ausgelesen werden
extern uint8 mcp2515_read_register(uint8 adress);
extern uint8 mcp2515_read_status(uint8 type);

// -------------------------------------------------------------------------
// Setzten/loeschen einzelner Bits
/*
 * Diese Funktionen laesst sich nur auf die Register BFPCTRL, 
 * TXRTSCTRL, CANCTRL, CNF1, CNF2, CNF3, CANINTE, CANINTF, EFLG,
 * TXB0CTRL, TXB1CTRL, TXB2CTRL, RXB0CTRL und RXB1CTRL anwenden.
 */
extern void mcp2515_bit_modify(uint8 adress, uint8 mask, uint8 data);

// -------------------------------------------------------------------------
// Liest bzw. schreibt eine CAN-Id aus/in Register von MCP2515

extern uint8 mcp2515_read_id(uint32 *id);

extern void mcp2515_write_id( const uint32 *id, uint8 extended );

// -------------------------------------------------------------------------
// verschickt Nachricht aus TX Register
// Parameter: buffer aus welchem TX Register
extern void RTS(unsigned char buffer);


#endif