// ----------------------------------------------------------------------------
/*
 * Copyright (c) 2007 Fabian Greif, Roboterclub Aachen e.V.
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: mcp2515_defs.h 6611 2008-07-14 09:58:16Z fabian $
 */
// ----------------------------------------------------------------------------

#ifndef	MCP2515_DEFS_H
#define	MCP2515_DEFS_H

/** \name	SPI Kommandos */
/*@{*/
#define SPI_RESET		0xC0
#define	SPI_READ		0x03
#define	SPI_READ_RX		0x90
#define	SPI_WRITE		0x02
#define	SPI_WRITE_TX	0x40
#define	SPI_WRITE_TX0	0x40
#define SPI_WRITE_TX0D	0x41
#define SPI_WRITE_TX1	0x42
#define	SPI_WRITE_TX1D	0x43
#define	SPI_WRITE_TX2	0x44
#define SPI_WRITE_TX2D	0x45
#define	SPI_RTS			0x80
#define SPI_RTS_TXB0    0x81
#define SPI_RTS_TXB1    0x82
#define SPI_RTS_TXB2    0x84
#define SPI_READ_STATUS	0xA0
#define	SPI_RX_STATUS	0xB0
#define	SPI_BIT_MODIFY	0x05
/*@}*/

/** \name	Adressen der Register des MCP2515
 *
 * Die Redundanten Adressen von z.B. dem Register CANSTAT 
 * (0x0E, 0x1E, 0x2E, ...) wurden dabei nicht mit aufgelistet.
 */
/*@{*/
#define RXF0SIDH	0x00		// FILTER 0 STANDARD IDENTIFIER HIGH
#define RXF0SIDL	0x01		// FILTER 0 STANDARD IDENTIFIER LOW
#define RXF0EID8	0x02		// FILTER 0 EXTENDED IDENTIFIER HIGH
#define RXF0EID0	0x03		// FILTER 0 EXTENDED IDENTIFIER LOW
#define RXF1SIDH	0x04		// FILTER 1 STANDARD IDENTIFIER HIGH
#define RXF1SIDL	0x05		// FILTER 1 STANDARD IDENTIFIER LOW
#define RXF1EID8	0x06		// FILTER 1 EXTENDED IDENTIFIER HIGH
#define RXF1EID0	0x07		// FILTER 1 EXTENDED IDENTIFIER LOW
#define RXF2SIDH	0x08		// FILTER 2 STANDARD IDENTIFIER HIGH
#define RXF2SIDL	0x09		// FILTER 2 STANDARD IDENTIFIER LOW
#define RXF2EID8	0x0A		// FILTER 2 EXTENDED IDENTIFIER HIGH
#define RXF2EID0	0x0B		// FILTER 2 EXTENDED IDENTIFIER LOW
#define BFPCTRL		0x0C		// RXnBF PIN CONTROL AND STATUS
#define TXRTSCTRL	0x0D		// TXnRTS PIN CONTROL AND STATUS REGISTER
#define CANSTAT		0x0E		// CAN Status Register
#define CANCTRL		0x0F		// CAN CONTROL REGISTER

#define RXF3SIDH	0x10		// FILTER 3 STANDARD IDENTIFIER HIGH
#define RXF3SIDL	0x11		// FILTER 3 STANDARD IDENTIFIER LOW
#define RXF3EID8	0x12		// FILTER 3 EXTENDED IDENTIFIER HIGH
#define RXF3EID0	0x13		// FILTER 3 EXTENDED IDENTIFIER LOW
#define RXF4SIDH	0x14		// FILTER 4 STANDARD IDENTIFIER HIGH
#define RXF4SIDL	0x15		// FILTER 4 STANDARD IDENTIFIER LOW
#define RXF4EID8	0x16		// FILTER 4 EXTENDED IDENTIFIER HIGH
#define RXF4EID0	0x17		// FILTER 4 EXTENDED IDENTIFIER LOW
#define RXF5SIDH	0x18		// FILTER 5 STANDARD IDENTIFIER HIGH
#define RXF5SIDL	0x19		// FILTER 5 STANDARD IDENTIFIER LOW
#define RXF5EID8	0x1A		// FILTER 5 EXTENDED IDENTIFIER HIGH
#define RXF5EID0	0x1B		// FILTER 5 EXTENDED IDENTIFIER LOW
#define TEC			0x1C		// Transmit Error Counter
#define REC         0x1D		// Receive	Error Counter

#define RXM0SIDH	0x20		// MASK 0 STANDARD IDENTIFIER HIGH
#define RXM0SIDL	0x21		// MASK 0 STANDARD IDENTIFIER LOW
#define RXM0EID8	0x22		// MASK 0 EXTENDED IDENTIFIER HIGH
#define RXM0EID0	0x23		// MASK 0 EXTENDED IDENTIFIER LOW
#define RXM1SIDH	0x24		// MASK 1 STANDARD IDENTIFIER HIGH
#define RXM1SIDL	0x25		// MASK 1 STANDARD IDENTIFIER LOW
#define RXM1EID8	0x26		// MASK 1 EXTENDED IDENTIFIER HIGH
#define RXM1EID0	0x27		// MASK 1 EXTENDED IDENTIFIER LOW
#define CNF3		0x28		// CONFIGURATION 1
#define CNF2		0x29		// CONFIGURATION 1
#define CNF1		0x2A		// CONFIGURATION 1
#define CANINTE		0x2B		// INTERRUPT ENABLE
#define CANINTF		0x2C		// INTERRUPT FLAG
#define EFLG		0x2D		// Error Flag

#define TXB0CTRL	0x30		// TRANSMIT BUFFER 0 CONTROL REGISTER
#define TXB0SIDH	0x31		// TRANSMIT BUFFER 0 STANDARD IDENTIFIER HIGH
#define TXB0SIDL	0x32		// TRANSMIT BUFFER 0 STANDARD IDENTIFIER LOW
#define TXB0EID8	0x33		// TRANSMIT BUFFER 0 EXTENDED IDENTIFIER HIGH
#define TXB0EID0	0x34		// TRANSMIT BUFFER 0 EXTENDED IDENTIFIER LOW
#define TXB0DLC		0x35		// TRANSMIT BUFFER 0 DATA LENGTH CODE
#define TXB0D0		0x36		// TRANSMIT BUFFER 0 DATA BYTE 0
#define TXB0D1		0x37		// TRANSMIT BUFFER 0 DATA BYTE 1
#define TXB0D2		0x38		// TRANSMIT BUFFER 0 DATA BYTE 2
#define TXB0D3		0x39		// TRANSMIT BUFFER 0 DATA BYTE 3
#define TXB0D4		0x3A		// TRANSMIT BUFFER 0 DATA BYTE 4
#define TXB0D5		0x3B		// TRANSMIT BUFFER 0 DATA BYTE 5
#define TXB0D6		0x3C		// TRANSMIT BUFFER 0 DATA BYTE 6
#define TXB0D7		0x3D		// TRANSMIT BUFFER 0 DATA BYTE 7

#define TXB1CTRL	0x40		// TRANSMIT BUFFER 1 CONTROL REGISTER
#define TXB1SIDH	0x41		// TRANSMIT BUFFER 1 STANDARD IDENTIFIER HIGH
#define TXB1SIDL	0x42		// TRANSMIT BUFFER 1 STANDARD IDENTIFIER LOW
#define TXB1EID8	0x43		// TRANSMIT BUFFER 1 EXTENDED IDENTIFIER HIGH
#define TXB1EID0	0x44		// TRANSMIT BUFFER 1 EXTENDED IDENTIFIER LOW
#define TXB1DLC		0x45		// TRANSMIT BUFFER 1 DATA LENGTH CODE
#define TXB1D0		0x46		// TRANSMIT BUFFER 1 DATA BYTE 0
#define TXB1D1		0x47		// TRANSMIT BUFFER 1 DATA BYTE 1
#define TXB1D2		0x48		// TRANSMIT BUFFER 1 DATA BYTE 2
#define TXB1D3		0x49		// TRANSMIT BUFFER 1 DATA BYTE 3
#define TXB1D4		0x4A		// TRANSMIT BUFFER 1 DATA BYTE 4
#define TXB1D5		0x4B		// TRANSMIT BUFFER 1 DATA BYTE 5
#define TXB1D6		0x4C		// TRANSMIT BUFFER 1 DATA BYTE 6
#define TXB1D7		0x4D		// TRANSMIT BUFFER 1 DATA BYTE 7

#define TXB2CTRL	0x50		// TRANSMIT BUFFER 2 CONTROL REGISTER
#define TXB2SIDH	0x51		// TRANSMIT BUFFER 2 STANDARD IDENTIFIER HIGH
#define TXB2SIDL	0x52		// TRANSMIT BUFFER 2 STANDARD IDENTIFIER LOW
#define TXB2EID8	0x53		// TRANSMIT BUFFER 2 EXTENDED IDENTIFIER HIGH
#define TXB2EID0	0x54		// TRANSMIT BUFFER 2 EXTENDED IDENTIFIER LOW
#define TXB2DLC		0x55		// TRANSMIT BUFFER 2 DATA LENGTH CODE
#define TXB2D0		0x56		// TRANSMIT BUFFER 2 DATA BYTE 0
#define TXB2D1		0x57		// TRANSMIT BUFFER 2 DATA BYTE 1
#define TXB2D2		0x58		// TRANSMIT BUFFER 2 DATA BYTE 2
#define TXB2D3		0x59		// TRANSMIT BUFFER 2 DATA BYTE 3
#define TXB2D4		0x5A		// TRANSMIT BUFFER 2 DATA BYTE 4
#define TXB2D5		0x5B		// TRANSMIT BUFFER 2 DATA BYTE 5
#define TXB2D6		0x5C		// TRANSMIT BUFFER 2 DATA BYTE 6
#define TXB2D7		0x5D		// TRANSMIT BUFFER 2 DATA BYTE 7

#define RXB0CTRL	0x60		// RECEIVE BUFFER 0 CONTROL
#define RXB0SIDH	0x61		// RECEIVE BUFFER 0 STANDARD IDENTIFIER HIGH
#define RXB0SIDL	0x62		// RECEIVE BUFFER 0 STANDARD IDENTIFIER LOW
#define RXB0EID8	0x63		// RECEIVE BUFFER 0 EXTENDED IDENTIFIER HIGH
#define RXB0EID0	0x64		// RECEIVE BUFFER 0 EXTENDED IDENTIFIER LOW
#define RXB0DLC		0x65		// RECEIVE BUFFER 0 STANDARD IDENTIFIER LOW
#define RXB0D0		0x66		// RECEIVE BUFFER 0 DATA BYTE 0
#define RXB0D1		0x67		// RECEIVE BUFFER 0 DATA BYTE 1
#define RXB0D2		0x68		// RECEIVE BUFFER 0 DATA BYTE 2
#define RXB0D3		0x69		// RECEIVE BUFFER 0 DATA BYTE 3
#define RXB0D4		0x6A		// RECEIVE BUFFER 0 DATA BYTE 4
#define RXB0D5		0x6B		// RECEIVE BUFFER 0 DATA BYTE 5
#define RXB0D6		0x6C		// RECEIVE BUFFER 0 DATA BYTE 6
#define RXB0D7		0x6D		// RECEIVE BUFFER 0 DATA BYTE 7

#define RXB1CTRL	0x70		// RECEIVE BUFFER 1 CONTROL
#define RXB1SIDH	0x71		// RECEIVE BUFFER 1 STANDARD IDENTIFIER HIGH
#define RXB1SIDL	0x72		// RECEIVE BUFFER 1 STANDARD IDENTIFIER LOW
#define RXB1EID8	0x73		// RECEIVE BUFFER 1 EXTENDED IDENTIFIER HIGH
#define RXB1EID0	0x74		// RECEIVE BUFFER 1 EXTENDED IDENTIFIER LOW
#define RXB1DLC		0x75		// RECEIVE BUFFER 1 STANDARD IDENTIFIER LOW
#define RXB1D0		0x76		// RECEIVE BUFFER 1 DATA BYTE 0
#define RXB1D1		0x77		// RECEIVE BUFFER 1 DATA BYTE 1		
#define RXB1D2		0x78		// RECEIVE BUFFER 1 DATA BYTE 2
#define RXB1D3		0x79		// RECEIVE BUFFER 1 DATA BYTE 3
#define RXB1D4		0x7A		// RECEIVE BUFFER 1 DATA BYTE 4
#define RXB1D5		0x7B		// RECEIVE BUFFER 1 DATA BYTE 5
#define RXB1D6		0x7C		// RECEIVE BUFFER 1 DATA BYTE 6
#define RXB1D7		0x7D		// RECEIVE BUFFER 1 DATA BYTE 7

/*@}*/

/** \name	Bitdefinition der verschiedenen Register */
/*@{*/

/** \brief	Bitdefinition von BFPCTRL */
#define B1BFS		5		// RX1BF Pin State bit (Digital Output mode only)
#define B0BFS		4		// RX0BF Pin State bit (Digital Output mode only)
#define B1BFE		3		// RX1BF Pin Function Enable bit
#define B0BFE		2		// RX0BF Pin Function Enable bit
#define B1BFM		1		// RX1BF Pin Operation mode bit
#define B0BFM		0		// RX0BF Pin Operation mode bit

/** \brief	Bitdefinition von TXRTSCTRL */
#define B2RTS		5		// TX2RTS Pin State bit
#define B1RTS		4		// TX1RTX Pin State bit
#define B0RTS		3		// TX0RTS Pin State bit
#define B2RTSM		2		// TX2RTS Pin mode bit
#define B1RTSM		1		// TX1RTS Pin mode bit
#define B0RTSM		0		// TX0RTS Pin mode bit

/** \brief	Bitdefinition von CANSTAT */
// Operation mode bits
#define OPMOD2		7
#define OPMOD1		6
#define OPMOD0		5
// Interrupt Flag Code bits
#define ICOD2		3
#define ICOD1		2
#define ICOD0		1

/** \brief	Bitdefinition von CANCTRL */
// Request Operation mode bits
#define REQOP2		7
#define REQOP1		6
#define REQOP0		5
#define ABAT		4		// Abort All Pending Transmissions bit
#define OSM			3		// One-Shot mode bit
#define CLKEN		2		// CLKOUT Pin Enable bit
// CLKOUT Pin Prescaler bits
#define CLKPRE1		1
#define CLKPRE0		0

/** \brief	Bitdefinition von CNF3 */
#define SOF			7		// Start-of-Frame signal bit
#define WAKFIL		6		// Wake-up Filter bit
// PS2 Length bits
#define PHSEG22		2
#define PHSEG21		1
#define PHSEG20		0

/** \brief	Bitdefinition von CNF2 */
#define BTLMODE		7		// PS2 Bit Time Length bit
#define SAM			6		// Sample Point Configuration bit
// PS1 Length bits
#define PHSEG12		5
#define PHSEG11		4
#define PHSEG10		3
// Propagation Segment Length bits
#define PHSEG2		2
#define PHSEG1		1
#define PHSEG0		0

/** \brief	Bitdefinition von CNF1 */
// Synchronization Jump Width Length bits
#define SJW1		7
#define SJW0		6
// Baud Rate Prescaler bits
#define BRP5		5
#define BRP4		4
#define BRP3		3
#define BRP2		2
#define BRP1		1
#define BRP0		0

/** \brief	Bitdefinition von CANINTE */
#define MERRE		7		// Message Error Interrupt Enable bit
#define WAKIE		6		// Wake-up Interrupt Enable bit
#define ERRIE		5		// Error Interrupt Enable bit (multiple sources in EFLG register)
#define TX2IE		4		// Transmit Buffer 2 Empty Interrupt Enable bit
#define TX1IE		3		// Transmit Buffer 1 Empty Interrupt Enable bit
#define TX0IE		2		// Transmit Buffer 0 Empty Interrupt Enable bit
#define RX1IE		1		// Receive Buffer 1 Full Interrupt Enable bit
#define RX0IE		0		// Receive Buffer 0 Full Interrupt Enable bit

/** \brief	Bitdefinition von CANINTF */
#define MERRF		7		// Message Error Interrupt Flag bit
#define WAKIF		6		// Wake-up Interrupt Flag bit
#define ERRIF		5		// Error Interrupt Flag bit (multiple sources in EFLG register)
#define TX2IF		4		// Transmit Buffer 2 Empty Interrupt Flag bit
#define TX1IF		3		// Transmit Buffer 1 Empty Interrupt Flag bit
#define TX0IF		2		// Transmit Buffer 0 Empty Interrupt Flag bit
#define RX1IF		1		// Receive Buffer 1 Full Interrupt Flag bit
#define RX0IF		0		// Receive Buffer 0 Full Interrupt Flag bit

/** \brief	Bitdefinition von EFLG */
#define RX1OVR		7		// Receive Buffer 1 Overflow Flag bit
#define RX0OVR		6		// Receive Buffer 0 Overflow Flag bit
#define TXB0		5		// Bus-Off Error Flag bit
#define TXEP		4		// Transmit Error-Passive Flag bit
#define RXEP		3		// Receive Error-Passive Flag bit
#define TXWAR		2		// Transmit Error Warning Flag bit
#define RXWAR		1		// Receive Error Warning Flag bit
#define EWARN		0		// Error Warning Flag bit

/** \brief	Bitdefinition von TXBnCTRL (n = 0, 1, 2) */
#define ABTF		6		// Message Aborted Flag bit
#define MLOA		5		// Message Lost Arbitration bit
#define TXERR		4		// Transmission Error Detected bit
#define TXREQ		3		// Message Transmit Request bit
// Transmit Buffer Priority bits
#define TXP1		1
#define TXP0		0

/** \brief	Bitdefinition von RXB0CTRL */
// Receive Buffer Operating mode bits
#define RXM1		6
#define RXM0		5
#define RXRTR		3		// Received Remote Transfer Request bit
#define BUKT		2		// Rollover Enable bit
#define BUKT1		1		// Read-only Copy of BUKT bit (used internally by the MCP2515)
#define FILHIT0		0		// Filter Hit bit  indicates which acceptance filter enabled reception of message

/** \brief	Bitdefinition von TXBnSIDL (n = 0, 1) */
#define	EXIDE		3		// Extended Identifier Enable bit

/**
 * \brief	Bitdefinition von RXB1CTRL
 * \see		RXM1, RXM0, RXRTR und FILHIT0 sind schon fuer RXB0CTRL definiert
 */
// Filter Hit bits - indicates which acceptance filter enabled reception of message
#define FILHIT2		2
#define FILHIT1		1
#define FILHIT0		0

/** \brief	Bitdefinition von RXBnSIDL (n = 0, 1) */
#define	SRR			4		// Standard Frame Remote Transmit Request bit
#define	IDE			3		// Extended Identifier Flag bit 

/**
 * \brief	Bitdefinition von RXBnDLC (n = 0, 1)
 * \see		TXBnDLC   (gleiche Bits)
 */
#define	RTR			6		// Extended Frame Remote Transmission Request bit
// Data Length Code bits
#define	DLC3		3
#define	DLC2		2
#define	DLC1		1
#define DLC0		0

/*@}*/
#endif	// MCP2515_DEFS_H
