/************************************************************************/
/*							INCLUDE FILES								*/
/************************************************************************/
#include <stdio.h>

#ifdef _WIN32
#include <windows.h>
#endif

#include "ftd2xx.h"
#include "libMPSSE_spi.h"
#include "can.h"
/************************************************************************/
/*							Helper Macros								*/
/************************************************************************/

#define APP_CHECK_STATUS(exp) {if(exp!=FT_OK){printf("%s:%d:%s(): status(0x%x) \
!= FT_OK\n",__FILE__, __LINE__, __FUNCTION__,exp);exit(1);}else{;}};
#define CHECK_NULL(exp){if(exp==NULL){printf("%s:%d:%s():  NULL expression \
encountered \n",__FILE__, __LINE__, __FUNCTION__);exit(1);}else{;}};

/************************************************************************/
/*							Globale Variable							*/
/************************************************************************/
FT_HANDLE ftHandle;

/************************************************************************/
/*				SPI Initialisieung und Kanal oeffnen						*/
/************************************************************************/
bool spi_init(uint32 CHANNEL_TO_OPEN, uint32 clockrate)
{
	uint32 channels;
	ChannelConfig channelConf;
	FT_STATUS status;
	FT_DEVICE_LIST_INFO_NODE devList;
	
	status = SPI_GetNumChannels(&channels);
	APP_CHECK_STATUS(status);
	
	if (channels > 0)
	{
		// Kofiguration des Bausteins FT2232H 
		channelConf.ClockRate = clockrate;			
		channelConf.LatencyTimer = 255;				// Wartezeit 255ms
		channelConf.configOptions = \
		// Daten werden bei steigender Flanke erfasst und bei Fallender uebertragen  // CS = ADBUS3
		SPI_CONFIG_OPTION_MODE0 | SPI_CONFIG_OPTION_CS_DBUS3 | SPI_CONFIG_OPTION_CS_ACTIVELOW ;	// CS ist Activelow
		channelConf.Pin = 0x00000000;	//FinalVal-FinalDir-InitVal-InitDir (for dir 0=in, 1=out)

		// Kanalinformationen erhalten 
		status = SPI_GetChannelInfo(CHANNEL_TO_OPEN, &devList); 
		APP_CHECK_STATUS(status);

		// ausgewaehlten Kanal oeffnen
		status = SPI_OpenChannel(CHANNEL_TO_OPEN, &ftHandle);
		APP_CHECK_STATUS(status);

		// Konfigurationen uebernehmen 
		status = SPI_InitChannel(ftHandle, &channelConf); 
		APP_CHECK_STATUS(status);

		return true;
	}
	else
	{
		printf("Kein Anschluss vorhanden!\n");
		return false;
	}
}
/************************************************************************/
/*							Kanal schlieen								*/
/************************************************************************/
void spi_closeChannel(void)
{
	SPI_CloseChannel(ftHandle);
}
/************************************************************************/
/*						SPI write und setzt CS							*/
/************************************************************************/
void spi_write(uint8 data)
{
	FT_STATUS status;
	uint32 sizeTransfered = 0;
	
	status = SPI_Write(ftHandle, &data, 1, &sizeTransfered, 
			SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE);
	APP_CHECK_STATUS(status);
}
/************************************************************************/
/*						SPI read und setzt CS							*/
/************************************************************************/
uint8 spi_read(void)
{
	FT_STATUS status;		
	uint8 buffer;
	uint32 sizeTransfered = 0;
		
	status = SPI_Read(ftHandle, &buffer, 1, &sizeTransfered,
		SPI_TRANSFER_OPTIONS_SIZE_IN_BYTES | SPI_TRANSFER_OPTIONS_CHIPSELECT_ENABLE);
	APP_CHECK_STATUS(status);

	return buffer;
}
/************************************************************************/
/*					setzt Chipselect zurueck								*/
/************************************************************************/
void Reset_CS(void)
{
	FT_STATUS status;
	uint8 buffer = 0;
	uint32 sizeTransfered = 0;

	status = SPI_Read(ftHandle, &buffer, 0, &sizeTransfered,
		SPI_TRANSFER_OPTIONS_SIZE_IN_BITS | SPI_TRANSFER_OPTIONS_CHIPSELECT_DISABLE);
	APP_CHECK_STATUS(status);
}

/************************************************************************/
/*								Dateiende								*/
/************************************************************************/